/* viajem.c */
#include <stdio.h>
#include <time.h>
#include <malloc.h>
#include <stdlib.h>
#include <graphics.h>
#include <conio.h>

#include "viagem.h"

int nave_z,
    velocidade,
    N_ESTRELAS;

void *aloca (long int li_numbytes)
{
  void *vp_estrutura;

  if ((vp_estrutura = malloc (li_numbytes)) == NULL) {
    printf ("Erro de alocacao de memoria.\n");
    exit (EXIT_FAILURE);
  }

  return (vp_estrutura);
}

void acende_estrela (t_estrela *s_estrela)
{
  putpixel (s_estrela->w_tela_x + (TELA_X / 2),
	    s_estrela->w_tela_y + (TELA_Y / 2), WHITE);
}

void apaga_estrela (t_estrela *s_estrela)
{
  putpixel (s_estrela->w_tela_x + (TELA_X / 2),
	    s_estrela->w_tela_y + (TELA_Y / 2), BLACK);
}

void traz_estrela (t_estrela *s_estrela)
{
  s_estrela->w_coor_z -= velocidade;
}

void randomize_posicao_estrela (t_estrela *s_estrela)
{
  if (velocidade < 0) {
    s_estrela->w_coor_x = rand () - (RAND_MAX / 2);
    s_estrela->w_coor_y = rand () - (RAND_MAX / 2);
    s_estrela->w_coor_z = NAVE_FOCO + 5;
  }
  else {
    s_estrela->w_coor_x = rand () - (RAND_MAX / 2);
    s_estrela->w_coor_y = rand () - (RAND_MAX / 2);
    s_estrela->w_coor_z = ESTRELA_DIST;
    s_estrela->w_coor_z = rand ();
  }
}

void projeta_estrela_tela (t_estrela *s_estrela)
{

  if (s_estrela->w_coor_z < nave_z || s_estrela->w_coor_z > ESTRELA_DIST)
    randomize_posicao_estrela (s_estrela);

  do {
    int i_div = ((nave_z - s_estrela->w_coor_z) / NAVE_FOCO);

    if (i_div != 0) {
      s_estrela->w_tela_x = s_estrela->w_coor_x / i_div;
      s_estrela->w_tela_y = s_estrela->w_coor_y / i_div;
    }
    else
      randomize_posicao_estrela (s_estrela);

    if (velocidade > 0)
      if (s_estrela->w_tela_x < -(TELA_X / 2) ||
	  s_estrela->w_tela_x > (TELA_X / 2) ||
	  s_estrela->w_tela_y < -(TELA_Y / 2) ||
	  s_estrela->w_tela_y > (TELA_Y / 2))
	randomize_posicao_estrela (s_estrela);
      else
	break;
    else
      break;
  } while (TRUE);
}

void inicializa_constelacao (t_estrela *s_constelacao)
{
  int w_ind;

  randomize ();
  for (w_ind = 0; w_ind < N_ESTRELAS; w_ind++)
    randomize_posicao_estrela (&s_constelacao [w_ind]);
}


void inicializa_grafico (void)
{
  int i_grf_drv, i_grf_mod, i_grf_err;

  i_grf_drv = DETECT;
  i_grf_mod = VGAHI;
  initgraph (&i_grf_drv, &i_grf_mod, ".");

  i_grf_err = graphresult ();
  if (i_grf_err != grOk) {
    printf ("Erro grafico\n");
    exit (EXIT_FAILURE);
  }
}

void mostra_estrelas (t_estrela *s_constelacao)
{
  int w_ind;

  for (w_ind = 0; w_ind < N_ESTRELAS; w_ind++) {
    apaga_estrela (&s_constelacao [w_ind]);
    traz_estrela (&s_constelacao [w_ind]);
    projeta_estrela_tela (&s_constelacao [w_ind]);
    acende_estrela (&s_constelacao [w_ind]);
  }
}


int main (int argv, char *argc [])
{
  if (argv != 2) {
    printf ("Viajem estrelas\n");
    exit (EXIT_FAILURE);
  }

  N_ESTRELAS = atoi (argc [1]);

  if (N_ESTRELAS < 1 || N_ESTRELAS > 32000) {
    printf ("Numero errado de estrelas\n");
    exit (EXIT_FAILURE);
  }

  t_estrela *s_constelacao = ALOCA (N_ESTRELAS, t_estrela);
  char c_tecla = 0x0;

  nave_z = NAVE_Z;

  inicializa_constelacao (s_constelacao);
  inicializa_grafico ();

  do {
    mostra_estrelas (s_constelacao);
    if (kbhit ()) {
      c_tecla = getch ();
      switch (c_tecla) {
	case '+': {
	  velocidade += 5;
	  break;
	}
	case '-': {
	  velocidade -= 5;
	  break;
	}
	case '0': {
	  velocidade = 0;
	  break;
	}
	case '1': {
	  velocidade = 10;
	  break;
	}
	case '2': {
	  velocidade = 20;
	  break;
	}
	case '3': {
	  velocidade = 30;
	  break;
	}
	case '4': {
	  velocidade = 40;
	  break;
	}
	case '5': {
	  velocidade = 50;
	  break;
	}
	case '6': {
	  velocidade = 60;
	  break;
	}
	case '7': {
	  velocidade = 70;
	  break;
	}
	case '8': {
	  velocidade = 80;
	  break;
	}
	case '9': {
	  velocidade = 90;
	  break;
	}
      }
    }
  } while (c_tecla != ' ');

  closegraph ();
  printf ("Carlos FBC\n");
}